@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2025 Cppcheck team.
+ * Copyright (C) 2007-2026 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@
 #include "projectfile.h"
 #include "projectfiledialog.h"
 #include "report.h"
+#include "resultstree.h"
 #include "resultsview.h"
 #include "scratchpad.h"
 #include "settings.h"
@@ -51,6 +52,8 @@
 #include "threadhandler.h"
 #include "threadresult.h"
 #include "translationhandler.h"
+#include "utils.h"
+#include "threaddetails.h"
 
 #include "ui_mainwindow.h"
 
@@ -181,6 +184,7 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
     connect(mUI->mActionShowHidden, &QAction::triggered, mUI->mResults, &ResultsView::showHiddenResults);
     connect(mUI->mActionViewStats, &QAction::triggered, this, &MainWindow::showStatistics);
     connect(mUI->mActionLibraryEditor, &QAction::triggered, this, &MainWindow::showLibraryEditor);
+    connect(mUI->mActionShowThreadDetails, &QAction::triggered, this, &MainWindow::showThreadDetails);
 
     connect(mUI->mActionReanalyzeModified, &QAction::triggered, this, &MainWindow::reAnalyzeModified);
     connect(mUI->mActionReanalyzeAll, &QAction::triggered, this, &MainWindow::reAnalyzeAll);
@@ -233,10 +237,13 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
     loadSettings();
 
     mThread->initialize(mUI->mResults);
-    if (mProjectFile)
+    if (mProjectFile) {
+        enableProjectActions(true);
         formatAndSetTitle(tr("Project:") + ' ' + mProjectFile->getFilename());
-    else
+    } else {
+        enableProjectActions(false);
         formatAndSetTitle();
+    }
 
     mUI->mActionComplianceReport->setVisible(isCppcheckPremium());
 
@@ -245,7 +252,6 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
     mUI->mActionPrint->setShortcut(QKeySequence::Print);
     enableResultsButtons();
     enableProjectOpenActions(true);
-    enableProjectActions(false);
 
     // Must setup MRU menu before CLI param handling as it can load a
     // project file and update MRU menu.
@@ -266,9 +272,6 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
         handleCLIParams(args);
     }
 
-    mUI->mActionCloseProjectFile->setEnabled(mProjectFile != nullptr);
-    mUI->mActionEditProjectFile->setEnabled(mProjectFile != nullptr);
-
     for (int i = 0; i < mPlatforms.getCount(); i++) {
         PlatformData platform = mPlatforms.mPlatforms[i];
         auto *action = new QAction(this);
@@ -295,6 +298,7 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
     mUI->mActionC11->setActionGroup(mCStandardActions);
     //mUI->mActionC17->setActionGroup(mCStandardActions);
     //mUI->mActionC23->setActionGroup(mCStandardActions);
+    //mUI->mActionC2Y->setActionGroup(mCStandardActions);
 
     mUI->mActionCpp03->setActionGroup(mCppStandardActions);
     mUI->mActionCpp11->setActionGroup(mCppStandardActions);
@@ -421,6 +425,7 @@ void MainWindow::loadSettings()
     mUI->mActionC11->setChecked(standards.c == Standards::C11);
     //mUI->mActionC17->setChecked(standards.c == Standards::C17);
     //mUI->mActionC23->setChecked(standards.c == Standards::C23);
+    //mUI->mActionC2Y->setChecked(standards.c == Standards::C2Y);
     standards.setCPP(mSettings->value(SETTINGS_STD_CPP, QString()).toString().toStdString());
     mUI->mActionCpp03->setChecked(standards.cpp == Standards::CPP03);
     mUI->mActionCpp11->setChecked(standards.cpp == Standards::CPP11);
@@ -522,6 +527,8 @@ void MainWindow::saveSettings() const
     //    mSettings->setValue(SETTINGS_STD_C, "C17");
     //if (mUI->mActionC23->isChecked())
     //    mSettings->setValue(SETTINGS_STD_C, "C23");
+    //if (mUI->mActionC2Y->isChecked())
+    //    mSettings->setValue(SETTINGS_STD_C, "C2Y");
 
     if (mUI->mActionCpp03->isChecked())
         mSettings->setValue(SETTINGS_STD_CPP, "C++03");
@@ -591,6 +598,7 @@ void MainWindow::doAnalyzeProject(ImportProject p, const bool checkLibrary, cons
     }
 
     mUI->mResults->clear(true);
+    mUI->mResults->setResultsSource(ResultsTree::ResultsSource::Analysis);
     mThread->clearFiles();
 
     mUI->mResults->checkingStarted(p.fileSettings.size());
@@ -612,7 +620,7 @@ void MainWindow::doAnalyzeProject(ImportProject p, const bool checkLibrary, cons
     if (!checkSettings.buildDir.empty()) {
         checkSettings.loadSummaries();
         std::list<std::string> sourcefiles;
-        AnalyzerInformation::writeFilesTxt(checkSettings.buildDir, sourcefiles, checkSettings.userDefines, p.fileSettings);
+        AnalyzerInformation::writeFilesTxt(checkSettings.buildDir, sourcefiles, p.fileSettings);
     }
 
     //mThread->SetanalyzeProject(true);
@@ -654,6 +662,7 @@ void MainWindow::doAnalyzeFiles(const QStringList &files, const bool checkLibrar
     QStringList fileNames = pathList.getFileList();
 
     mUI->mResults->clear(true);
+    mUI->mResults->setResultsSource(ResultsTree::ResultsSource::Analysis);
     mThread->clearFiles();
 
     if (fileNames.isEmpty()) {
@@ -693,7 +702,7 @@ void MainWindow::doAnalyzeFiles(const QStringList &files, const bool checkLibrar
         std::transform(fileNames.cbegin(), fileNames.cend(), std::back_inserter(sourcefiles), [](const QString& s) {
             return s.toStdString();
         });
-        AnalyzerInformation::writeFilesTxt(checkSettings.buildDir, sourcefiles, checkSettings.userDefines, {});
+        AnalyzerInformation::writeFilesTxt(checkSettings.buildDir, sourcefiles, {});
     }
 
     mThread->setCheckFiles(true);
@@ -721,14 +730,17 @@ void MainWindow::analyzeCode(const QString& code, const QString& filename)
             mUI->mResults, SLOT(debugError(ErrorItem)));
 
     // Create CppCheck instance
-    CppCheck cppcheck(checkSettings, supprs, result, true, nullptr);
+    CppCheck cppcheck(checkSettings, supprs, result, nullptr, true, nullptr);
 
     // Check
     checkLockDownUI();
     clearResults();
     mUI->mResults->checkingStarted(1);
-    // TODO: apply enforcedLanguage?
-    cppcheck.check(FileWithDetails(filename.toStdString(), Path::identify(filename.toStdString(), false), 0), code.toStdString());
+    {
+        const std::string code_s = code.toStdString();
+        // TODO: apply enforcedLanguage?
+        cppcheck.checkBuffer(FileWithDetails(filename.toStdString(), Path::identify(filename.toStdString(), false), 0), code_s.data(), code_s.size());
+    }
     analysisDone();
 
     // Expand results
@@ -1006,9 +1018,6 @@ bool MainWindow::tryLoadLibrary(Library &library, const QString& filename)
         case Library::ErrorCode::UNKNOWN_ELEMENT:
             errmsg = tr("Unknown element");
             break;
-        default:
-            errmsg = tr("Unknown issue");
-            break;
         }
         if (!error.reason.empty())
             errmsg += " '" + QString::fromStdString(error.reason) + "'";
@@ -1062,6 +1071,7 @@ bool MainWindow::getCppcheckSettings(Settings& settings, Suppressions& supprs)
 
     settings.exename = QCoreApplication::applicationFilePath().toStdString();
     settings.templateFormat = "{file}:{line}:{column}: {severity}:{inconclusive:inconclusive:} {message} [{id}]";
+    settings.reportProgress = 10;
 
     // default to --check-level=normal for GUI for now
     settings.setCheckLevel(Settings::CheckLevel::normal);
@@ -1082,6 +1092,8 @@ bool MainWindow::getCppcheckSettings(Settings& settings, Suppressions& supprs)
             return false;
         }
 
+        settings.premium = startsWith(settings.cppcheckCfgProductName, "Cppcheck Premium");
+
         const auto cfgAddons = settings.addons;
         settings.addons.clear();
         for (const std::string& addon : cfgAddons) {
@@ -1125,13 +1137,9 @@ bool MainWindow::getCppcheckSettings(Settings& settings, Suppressions& supprs)
             supprs.nomsg.addSuppression(suppression); // TODO: check result
         }
 
-        // Only check the given -D configuration
-        if (!defines.isEmpty())
-            settings.maxConfigs = 1;
-
         // If importing a project, only check the given configuration
-        if (!mProjectFile->getImportProject().isEmpty())
-            settings.checkAllConfigurations = false;
+        if (mProjectFile->getImportProject().endsWith("json", Qt::CaseInsensitive))
+            settings.maxConfigsProject = 1;
 
         const QString &buildDir = fromNativePath(mProjectFile->getBuildDir());
         if (!buildDir.isEmpty()) {
@@ -1145,8 +1153,11 @@ bool MainWindow::getCppcheckSettings(Settings& settings, Suppressions& supprs)
 
         const QString platform = mProjectFile->getPlatform();
         if (platform.endsWith(".xml")) {
-            const QString applicationFilePath = QCoreApplication::applicationFilePath();
-            settings.platform.loadFromFile(applicationFilePath.toStdString().c_str(), platform.toStdString());
+            const std::vector<std::string> paths = {
+                Path::getCurrentPath(), // TODO: do we want to look in CWD?
+                QCoreApplication::applicationFilePath().toStdString(),
+            };
+            settings.platform.loadFromFile(paths, platform.toStdString());
         } else {
             for (int i = Platform::Type::Native; i <= Platform::Type::Unix64; i++) {
                 const auto p = static_cast<Platform::Type>(i);
@@ -1225,7 +1236,7 @@ bool MainWindow::getCppcheckSettings(Settings& settings, Suppressions& supprs)
     settings.debugwarnings = mSettings->value(SETTINGS_SHOW_DEBUG_WARNINGS, false).toBool();
     settings.quiet = false;
     settings.verbose = true;
-    settings.force = mSettings->value(SETTINGS_CHECK_FORCE, 1).toBool();
+    settings.force = mSettings->value(SETTINGS_CHECK_FORCE, 0).toBool();
     settings.outputFormat = Settings::OutputFormat::text;
     settings.jobs = mSettings->value(SETTINGS_CHECK_THREADS, 1).toInt();
     settings.certainty.setEnabled(Certainty::inconclusive, mSettings->value(SETTINGS_INCONCLUSIVE_ERRORS, false).toBool());
@@ -1489,6 +1500,7 @@ void MainWindow::loadResults(const QString &selectedFile)
         closeProjectFile();
     mIsLogfileLoaded = true;
     mUI->mResults->clear(true);
+    mUI->mResults->setResultsSource(ResultsTree::ResultsSource::Log);
     mUI->mActionReanalyzeModified->setEnabled(false);
     mUI->mActionReanalyzeAll->setEnabled(false);
     mUI->mResults->readErrorsXml(selectedFile);
@@ -1524,11 +1536,10 @@ void MainWindow::enableCheckButtons(bool enable)
 
 void MainWindow::enableResultsButtons()
 {
-    const bool enabled = mUI->mResults->hasResults();
-    mUI->mActionClearResults->setEnabled(enabled);
-    mUI->mActionSave->setEnabled(enabled);
-    mUI->mActionPrint->setEnabled(enabled);
-    mUI->mActionPrintPreview->setEnabled(enabled);
+    mUI->mActionClearResults->setEnabled(mUI->mResults->hasResults());
+    mUI->mActionSave->setEnabled(true);
+    mUI->mActionPrint->setEnabled(true);
+    mUI->mActionPrintPreview->setEnabled(true);
 }
 
 void MainWindow::showStyle(bool checked)
@@ -1707,7 +1718,7 @@ void MainWindow::complianceReport()
     }
 
     QTemporaryFile tempResults;
-    tempResults.open();
+    (void)tempResults.open(); // TODO: check result
     tempResults.close();
 
     mUI->mResults->save(tempResults.fileName(), Report::XMLV2, mCppcheckCfgProductName);
@@ -1717,7 +1728,9 @@ void MainWindow::complianceReport()
 }
 
 void MainWindow::resultsAdded()
-{}
+{
+    enableResultsButtons();
+}
 
 void MainWindow::toggleMainToolBar()
 {
@@ -1744,6 +1757,7 @@ void MainWindow::formatAndSetTitle(const QString &text)
         nameWithVersion += " (" + extraVersion + ")";
     }
 
+    // TODO: should not contain the version
     if (!mCppcheckCfgProductName.isEmpty())
         nameWithVersion = mCppcheckCfgProductName;
 
@@ -1784,7 +1798,6 @@ void MainWindow::stopAnalysis()
 {
     mThread->stop();
     mUI->mResults->stopAnalysis();
-    mUI->mResults->disableProgressbar();
     const QString &lastResults = getLastResults();
     if (!lastResults.isEmpty()) {
         mUI->mResults->updateFromOldReport(lastResults);
@@ -1838,8 +1851,10 @@ void MainWindow::loadProjectFile(const QString &filePath)
     addProjectMRU(filePath);
 
     mIsLogfileLoaded = false;
-    mUI->mActionCloseProjectFile->setEnabled(true);
-    mUI->mActionEditProjectFile->setEnabled(true);
+    mUI->mResults->setResultsSource(ResultsTree::ResultsSource::Analysis);
+    mUI->mActionReanalyzeModified->setEnabled(true);
+    mUI->mActionReanalyzeAll->setEnabled(true);
+    enableProjectActions(true);
     delete mProjectFile;
     mProjectFile = new ProjectFile(filePath, this);
     mProjectFile->setActiveProject();
@@ -1973,6 +1988,12 @@ void MainWindow::analyzeProject(const ProjectFile *projectFile, const QStringLis
                 break;
             }
 
+            if (!p.errors.empty())
+                errorMessage += ": \n";
+
+            for (const auto &error : p.errors)
+                errorMessage += "\n - " + QString::fromStdString(error);
+
             if (!errorMessage.isEmpty()) {
                 QMessageBox msg(QMessageBox::Critical,
                                 tr("Cppcheck"),
@@ -2091,6 +2112,20 @@ void MainWindow::showLibraryEditor()
     libraryDialog.exec();
 }
 
+void MainWindow::showThreadDetails()
+{
+    if (ThreadDetails::instance())
+        return;
+    auto* threadDetails = new ThreadDetails(this);
+    connect(mThread, &ThreadHandler::threadDetailsUpdated,
+            threadDetails, &ThreadDetails::threadDetailsUpdated, Qt::QueuedConnection);
+    connect(mThread, &ThreadHandler::progress,
+            threadDetails, &ThreadDetails::progress, Qt::QueuedConnection);
+    threadDetails->setAttribute(Qt::WA_DeleteOnClose);
+    threadDetails->show();
+    mThread->emitThreadDetailsUpdated();
+}
+
 void MainWindow::filterResults()
 {
     mUI->mResults->filterResults(mLineEditFilter->text());
@@ -2248,12 +2283,12 @@ static int getVersion(const QString& nameWithVersion) {
         }
         else if (c == '.') {
             ++dot;
-            ret = ret * 1000 + v;
+            ret = (ret * 1000) + v;
             v = 0;
         } else if (c >= '0' && c <= '9')
-            v = v * 10 + (c.toLatin1() - '0');
+            v = (v * 10) + (c.toLatin1() - '0');
     }
-    ret = ret * 1000 + v;
+    ret = (ret * 1000) + v;
     while (dot < 2) {
         ++dot;
         ret *= 1000;
@@ -2271,7 +2306,9 @@ void MainWindow::replyFinished(QNetworkReply *reply) {
     const QString str = reply->readAll();
     qDebug() << "Response: " << str;
     if (reply->url().fileName() == "version.txt") {
+        // TODO: lacks extra version
         QString nameWithVersion = QString("Cppcheck %1").arg(CppCheck::version());
+        // TODO: this should not contain the version
         if (!mCppcheckCfgProductName.isEmpty())
             nameWithVersion = mCppcheckCfgProductName;
         const int appVersion = getVersion(nameWithVersion);
