@@ -1,6 +1,6 @@
 /* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2025 Cppcheck team.
+ * Copyright (C) 2007-2026 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,8 +45,6 @@ struct newInstantiation;
 
 /** @brief Simplify templates from the preprocessed and partially simplified code. */
 class CPPCHECKLIB TemplateSimplifier {
-    friend class TestSimplifyTemplate;
-
 public:
     explicit TemplateSimplifier(Tokenizer &tokenizer);
 
@@ -267,6 +265,7 @@ public:
     static Token *findTemplateDeclarationEnd(Token *tok);
     static const Token *findTemplateDeclarationEnd(const Token *tok);
 
+protected:
     /**
      * Match template declaration/instantiation
      * @param instance template instantiation
@@ -277,6 +276,7 @@ public:
      */
     static bool instantiateMatch(const Token *instance, std::size_t numberOfArguments, bool variadic, const char patternAfter[]);
 
+public: // TODO: only needs to be public for tests
     /**
      * Match template declaration/instantiation
      * @param tok The ">" token e.g. before "class"
@@ -285,6 +285,7 @@ public:
      */
     int getTemplateNamePosition(const Token *tok);
 
+private:
     /**
      * Get class template name position
      * @param tok The ">" token e.g. before "class"
@@ -309,6 +310,7 @@ public:
      * */
     static bool getTemplateNamePositionTemplateVariable(const Token *tok, int &namepos);
 
+public:
     /**
      * Simplify templates
      * @param maxtime time when the simplification should be stopped
@@ -324,6 +326,7 @@ public:
      */
     static bool simplifyNumericCalculations(Token *tok, bool isTemplate = true);
 
+private:
     /**
      * Simplify constant calculations such as "1+2" => "3".
      * This also performs simple cleanup of parentheses etc.
@@ -338,7 +341,6 @@ public:
      */
     void simplifyTemplateArgs(Token *start, const Token *end, std::vector<newInstantiation>* newInst = nullptr);
 
-private:
     /**
      * Get template declarations
      * @return true if code has templates.
@@ -445,6 +447,7 @@ private:
                               const std::list<std::string> &typeStringsUsedInTemplateInstantiation,
                               const std::string &newName);
 
+protected:
     /**
      * @brief TemplateParametersInDeclaration
      * @param tok  template < typename T, typename S >
@@ -456,6 +459,7 @@ private:
         const Token * tok,
         std::vector<const Token *> & typeParametersInDeclaration);
 
+private:
     /**
      * Remove a specific "template < ..." template class/function
      */
