@@ -1,6 +1,6 @@
 /* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2025 Cppcheck team.
+ * Copyright (C) 2007-2026 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -696,6 +696,8 @@ private:
         std::string filename;
         int lineNumber;
         int column;
+        int tagLine{-1};
+        int tagColumn{-1};
         bool used;
         bool isFunctionPointer;
         std::vector<TypedefToken> typedefInfoTokens;
